use std::fmt::{self, Display};

pub type Result<T> = std::result::Result<T, Error>;

#[derive(Debug, Eq, PartialEq)]
pub enum Error {
    Message(String),
    Eof,
    TrailingBytes,
    UnknownMsg,
    UnexpectedType,
    UnknownEnum(u64),
    SequenceTooLong,
    MissingResult,
    InvalidQmiError(u16),
    // Errors defined by QMI protocol
    MalformedMessage,
    NoMemory,
    Internal,
    Aborted,
    ClientIdsExhausted,
    UnabortableTransaction,
    InvalidClientId,
    NoThresholdsProvided,
    InvalidHandle,
    InvalidProfile,
    InvalidPinId,
    IncorrectPin,
    NoNetworkFound,
    CallFailed,
    OutOfCall,
    NotProvisioned,
    MissingArgument,
    ArgumentTooLong,
    InvalidTransactionId,
    DeviceInUse,
    NetworkUnsupported,
    DeviceUnsupported,
    NoEffect,
    NoFreeProfile,
    InvalidPdpType,
    InvalidTechnologyPreference,
    InvalidProfileType,
    InvalidServiceType,
    InvalidRegisterAction,
    InvalidPsAttachAction,
    AuthenticationFailed,
    PinBlocked,
    PinAlwaysBlocked,
    UimUninitialized,
    MaximumQosRequestsInUse,
    IncorrectFlowFilter,
    NetworkQosUnaware,
    InvalidQosId,
    RequestedNumberUnsupported,
    InterfaceNotFound,
    FlowSuspended,
    InvalidDataFormat,
    GeneralError,
    UnknownError,
    InvalidArgument,
    InvalidIndex,
    NoEntry,
    DeviceStorageFull,
    DeviceNotReady,
    NetworkNotReady,
    WmsCauseCode,
    WmsMessageNotSent,
    WmsMessageDeliveryFailure,
    WmsInvalidMessageId,
    WmsEncoding,
    AuthenticationLock,
    InvalidTransition,
    NotMcastInterface,
    MaximumMcastRequestsInUse,
    InvalidMcastHandle,
    InvalidIpFamilyPreference,
    SessionInactive,
    SessionInvalid,
    SessionOwnership,
    InsufficientResources,
    Disabled,
    InvalidOperation,
    InvalidQmiCommand,
    WmsTPduType,
    WmsSmscAddress,
    InformationUnavailable,
    SegmentTooLong,
    SegmentOrder,
    BundlingNotSupported,
    OperationPartialFailure,
    PolicyMismatch,
    SimFileNotFound,
    ExtendedInternal,
    AccessDenied,
    HardwareRestricted,
    AckNotSent,
    InjectTimeout,
    IncompatibleState,
    FdnRestrict,
    SupsFailureCase,
    NoRadio,
    NotSupported,
    NoSubscription,
    CardCallControlFailed,
    NetworkAborted,
    MsgBlocked,
    InvalidSessionType,
    InvalidPbType,
    NoSim,
    PbNotReady,
    PinRestriction,
    Pin1Restriction,
    PukRestriction,
    Puk2Restriction,
    PbAccessRestricted,
    PbDeleteInProgress,
    PbTextTooLong,
    PbNumberTooLong,
    PbHiddenKeyRestriction,
    PbNotAvailable,
    DeviceMemoryError,
    NoPermission,
    TooSoon,
    TimeNotAcquired,
    OperationInProgress,
    FwWriteFailed,
    FwInfoReadFailed,
    FwFileNotFound,
    FwDirNotFound,
    FwAlreadyActivated,
    FwCannotGenericImage,
    FwFileOpenFailed,
    FwUpdateDiscontinuousFrame,
    FwUpdateFailed,
    CatEventRegistrationFailed,
    CatInvalidTerminalResponse,
    CatInvalidEnvelopeCommand,
    CatEnvelopeCommandBusy,
    CatEnvelopeCommandFailed,
}

impl Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            Error::Message(msg) => f.write_str(msg),
            Error::Eof => write!(f, "unexpected end of sequence"),
            Error::TrailingBytes => write!(f, "trailing bytes"),
            Error::UnknownMsg => write!(f, "unknown message"),
            Error::UnexpectedType => write!(f, "unexpected message type"),
            Error::UnknownEnum(v) => write!(f, "unknown enumeration value: {v}"),
            Error::SequenceTooLong => write!(f, "sequence too long"),
            Error::MissingResult => write!(f, "missing result"),
            Error::InvalidQmiError(v) => write!(f, "unknown QMI error: {v}"),
            _ => write!(f, "{:#?}", self),
        }
    }
}

impl std::error::Error for Error {}
